@@ -3,9 +3,15 @@ use crate::{
     metadata::Metadata,
     shm::{shm_open_read, shm_open_write, shm_unlink, SHM_DIR},
 };
+use aes_gcm::{
+    aead::{Aead, NewAead},
+    Aes256Gcm, Key, Nonce,
+};
 use chrono::Utc;
+use log::warn;
 use memmap2::{Mmap, MmapMut};
 use named_lock::NamedLock;
+use rand::{seq::SliceRandom, thread_rng};
 use serde::{de::DeserializeOwned, Serialize};
 use sha2::{Digest, Sha224};
 use std::{path::PathBuf, time::Duration};
@@ -13,17 +19,46 @@ use std::{path::PathBuf, time::Duration};
 const METADATA_SUFFIX: &str = "metadata";
 const SHMAP_PREFIX: &str = "shmap";
 
-#[derive(Clone, Copy)]
-pub struct Shmap {}
+#[derive(Clone)]
+pub struct Shmap {
+    cipher: Option<Aes256Gcm>,
+}
+
+impl Default for Shmap {
+    /// Default is Shmap without encryption.
+    fn default() -> Self {
+        Self::new()
+    }
+}
 
 impl Shmap {
-    pub fn new() -> Result<Self, ShmapError> {
-        fdlimit::raise_fd_limit();
-        let shmap = Shmap {};
-        shmap.clean()?;
-        Ok(shmap)
+    /// Initialize Shmap with no TTL or encryption.
+    pub fn new() -> Self {
+        Shmap::_new(None)
     }
 
+    /// Initialize Shmap with AES256 encryption key (random bytes).
+    pub fn new_with_encryption(encryption_key: &[u8; 32]) -> Self {
+        Shmap::_new(Some(encryption_key))
+    }
+
+    fn _new(encryption_key: Option<&[u8; 32]>) -> Self {
+        fdlimit::raise_fd_limit();
+
+        // If an encryption key was provided, create and store a `cipher` for AES256-GCM
+        let cipher = encryption_key.map(|key| {
+            let key = Key::from_slice(key);
+            Aes256Gcm::new(key)
+        });
+
+        let shmap = Shmap { cipher };
+        if let Err(e) = shmap.clean() {
+            warn!("Error while cleaning shmap keys: {}", e)
+        }
+        shmap
+    }
+
+    /// Get an item value by its key, and deserialize it (using `bincode`) to T.
     pub fn get<T>(&self, key: &str) -> Result<Option<T>, ShmapError>
     where
         T: DeserializeOwned,
@@ -48,27 +83,46 @@ impl Shmap {
             return Ok(None);
         }
 
-        self._get(&sanitized_key)
+        self.get_deserialize(&sanitized_key)
     }
 
     fn get_metadata(&self, key: &str) -> Result<Option<Metadata>, ShmapError> {
-        let sanitize_metadata_key = sanitize_metadata_key(key);
-        self._get(&sanitize_metadata_key)
+        let sanitized_metadata_key = sanitize_metadata_key(key);
+        self.get_deserialize(&sanitized_metadata_key)
     }
 
-    fn _get<T>(&self, sanitized_key: &str) -> Result<Option<T>, ShmapError>
+    fn get_deserialize<T>(&self, sanitized_key: &str) -> Result<Option<T>, ShmapError>
     where
         T: DeserializeOwned,
     {
+        match self._get(sanitized_key)? {
+            Some(bytes) => {
+                let (value, _): (T, usize) =
+                    bincode::serde::decode_from_slice(&bytes, bincode::config::standard())?;
+                Ok(Some(value))
+            }
+            None => Ok(None),
+        }
+    }
+
+    /// Get an item by its key, without deserialization, as bytes.
+    pub fn get_raw(&self, key: &str) -> Result<Option<Vec<u8>>, ShmapError> {
+        let sanitized_key = sanitize_key(key);
+        self._get(&sanitized_key)
+    }
+
+    fn _get(&self, sanitized_key: &str) -> Result<Option<Vec<u8>>, ShmapError> {
         let lock = NamedLock::with_path(PathBuf::from(SHM_DIR).join(sanitized_key))?;
         let guard = lock.lock()?;
 
+        // Read the item from shm
         let fd = match shm_open_read(sanitized_key) {
             Ok(fd) => fd,
             Err(e) => match e {
-                ShmapError::ShmNotFound => {
+                ShmapError::ShmFileNotFound => {
+                    // If the shm returns "file not found", return None
                     drop(guard);
-                    let _ = self._remove(sanitized_key);
+                    //let _ = self._remove(sanitized_key); // useless
                     return Ok(None);
                 }
                 e => return Err(e),
@@ -76,47 +130,90 @@ impl Shmap {
         };
         let mmap = unsafe { Mmap::map(fd) }?;
         if mmap.len() == 0 {
+            // If the value is empty, remove it and return None
             drop(guard);
             let _ = self._remove(sanitized_key);
             return Ok(None);
         }
-        let (value, _): (T, usize) =
-            bincode::serde::decode_from_slice(mmap.as_ref(), bincode::config::standard())?;
-        Ok(Some(value))
+
+        // If an encryption key was provided, decrypt the value
+        let bytes = if let Some(cipher) = &self.cipher {
+            let nonce = Nonce::from_slice(&mmap[..12]);
+            cipher.decrypt(nonce, &mmap[12..])?
+        } else {
+            mmap.to_vec()
+        };
+
+        Ok(Some(bytes))
     }
 
+    /// Insert a new item, using `bincode` serialization.
     pub fn insert<T>(&self, key: &str, value: T) -> Result<(), ShmapError>
     where
         T: Serialize,
     {
         let sanitized_key = sanitize_key(key);
-        self._insert(&sanitized_key, value)?;
-        self.insert_metadata(Metadata::new(key, None)?)
+        self.insert_serialize(&sanitized_key, value)?;
+        self.insert_metadata(Metadata::new(key, None, self.cipher.is_some())?)
     }
 
+    /// Insert a new item, using `bincode` serialization, with a TTL.
     pub fn insert_with_ttl<T>(&self, key: &str, value: T, ttl: Duration) -> Result<(), ShmapError>
     where
         T: Serialize,
     {
+        let sanitized_key = sanitize_key(key);
+        self.insert_serialize(&sanitized_key, value)?;
+        self.insert_metadata(Metadata::new(key, Some(ttl), self.cipher.is_some())?)
+    }
+
+    /// Insert a new item, without serialization, with a TTL.
+    pub fn insert_raw_with_ttl(
+        &self,
+        key: &str,
+        value: &[u8],
+        ttl: Duration,
+    ) -> Result<(), ShmapError> {
         let sanitized_key = sanitize_key(key);
         self._insert(&sanitized_key, value)?;
-        self.insert_metadata(Metadata::new(key, Some(ttl))?)
+        self.insert_metadata(Metadata::new(key, Some(ttl), self.cipher.is_some())?)
     }
 
     fn insert_metadata(&self, metadata: Metadata) -> Result<(), ShmapError> {
         let sanitize_metadata_key = sanitize_metadata_key(&metadata.key);
-        self._insert(&sanitize_metadata_key, metadata)
+        self.insert_serialize(&sanitize_metadata_key, metadata)
     }
 
-    fn _insert<T>(&self, sanitized_key: &str, value: T) -> Result<(), ShmapError>
+    fn insert_serialize<T>(&self, sanitized_key: &str, value: T) -> Result<(), ShmapError>
     where
         T: Serialize,
     {
         let bytes = bincode::serde::encode_to_vec(&value, bincode::config::standard())?;
+        self._insert(sanitized_key, &bytes)
+    }
+
+    /// Insert a new item, without serialization.
+    pub fn insert_raw(&self, key: &str, value: &[u8]) -> Result<(), ShmapError> {
+        let sanitized_key = sanitize_key(key);
+        self._insert(&sanitized_key, value)
+    }
+
+    fn _insert(&self, sanitized_key: &str, value: &[u8]) -> Result<(), ShmapError> {
+        // If an encryption key was provided, encrypt the value
+        let bytes = if let Some(cipher) = &self.cipher {
+            let mut nonce: Vec<u8> = (0..12).collect();
+            nonce.shuffle(&mut thread_rng());
+            let mut ciphertext = cipher.encrypt(Nonce::from_slice(nonce.as_slice()), value)?;
+            nonce.append(&mut ciphertext);
+            nonce
+        } else {
+            value.to_vec()
+        };
 
         let lock = NamedLock::with_path(PathBuf::from(SHM_DIR).join(sanitized_key))?;
         let guard = lock.lock()?;
 
+        // Insert the item to shm
         match || -> Result<(), ShmapError> {
             let fd = shm_open_write(sanitized_key, bytes.len())?;
             let mut mmap = unsafe { MmapMut::map_mut(fd) }?;
@@ -132,6 +229,7 @@ impl Shmap {
         }
     }
 
+    /// Remove an item by its key.
     pub fn remove(&self, key: &str) -> Result<(), ShmapError> {
         let sanitized_key = sanitize_key(key);
         self._remove(&sanitized_key)?;
@@ -147,18 +245,19 @@ impl Shmap {
         let lock = NamedLock::with_path(PathBuf::from(SHM_DIR).join(sanitized_key))?;
         let _guard = lock.lock()?;
 
-        let lock_file = std::env::var_os("TMPDIR")
-            .map(PathBuf::from)
-            .unwrap_or_else(|| PathBuf::from("/tmp"))
-            .join(format!("{}.lock", sanitized_key));
-        let _ = std::fs::remove_file(lock_file);
         shm_unlink(sanitized_key)?;
 
         Ok(())
     }
 
-    /// Clean expired items
-    pub fn clean(&self) -> Result<(), ShmapError> {
+    /// List available keys.
+    pub fn keys(&self) -> Result<Vec<String>, ShmapError> {
+        self.clean()
+    }
+
+    /// Clean expired items.
+    pub fn clean(&self) -> Result<Vec<String>, ShmapError> {
+        let mut keys = Vec::<String>::new();
         let read_dir = std::fs::read_dir(PathBuf::from(SHM_DIR))?;
         read_dir.into_iter().for_each(|dir_entry_res| {
             if let Ok(dir_entry) = dir_entry_res {
@@ -166,23 +265,30 @@ impl Shmap {
                 if filename.starts_with(SHMAP_PREFIX) && !filename.ends_with(METADATA_SUFFIX) {
                     let metadata_filename =
                         format!("{}.{}", dir_entry.path().to_string_lossy(), METADATA_SUFFIX);
-                    match self._get::<Metadata>(&metadata_filename) {
+                    match self.get_deserialize::<Metadata>(&metadata_filename) {
                         Ok(Some(metadata)) => match metadata.expiration {
                             Some(expiration) => {
-                                let keep = Utc::now().le(&expiration);
-                                if !keep {
+                                if Utc::now().gt(&expiration) {
                                     // Expired, remove item and metadata
                                     let _ = self._remove(&filename);
                                     let _ = self._remove(&metadata_filename);
+                                } else {
+                                    // Not expired, add to list
+                                    keys.push(metadata.key);
                                 }
                             }
-                            None => {}
+                            None => {
+                                // Not expiration, add to list
+                                keys.push(metadata.key);
+                            }
                         },
                         Ok(None) => {
                             // Item exists, but metadata not found, remove item
                             let _ = self._remove(&filename);
                         }
-                        Err(_) => {}
+                        Err(_) => {
+                            // Can't deserialized metadata or something else happens
+                        }
                     }
                 } else if filename.starts_with(SHMAP_PREFIX) && filename.ends_with(METADATA_SUFFIX)
                 {
@@ -196,7 +302,7 @@ impl Shmap {
                 }
             }
         });
-        Ok(())
+        Ok(keys)
     }
 }
 
@@ -216,14 +322,14 @@ mod tests {
 
     #[test]
     fn test_metadatas_presence() {
-        let shmap = Shmap::new().unwrap();
+        let shmap = Shmap::new();
         let key = rand_string(10);
         let value = rand_string(50);
 
         shmap.insert(&key, value).unwrap();
         let _ = shmap.get_metadata(&key).unwrap().unwrap();
 
-        let shmap = Shmap::new().unwrap();
+        let shmap = Shmap::new();
         shmap.remove(&key).unwrap();
         let should_be_none = shmap.get_metadata(&key).unwrap();
         assert!(should_be_none.is_none());
